// subor ModelSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.defobj.SymbolImpl;

import swarm.defobj.FArguments;
import swarm.defobj.FArgumentsImpl;
import swarm.defobj.FCall;
import swarm.defobj.FCallImpl;

import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.FActionForEach;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.VarProbe;
import swarm.objectbase.MessageProbe;
import swarm.objectbase.EmptyProbeMapImpl;

import java.util.LinkedList;
import java.util.List;

import swarm.space.Grid2d;
import swarm.space.Grid2dImpl;

public final class ModelSwarm extends SwarmImpl
{
	private boolean debug = true;
	
	private int worldXSize, worldYSize;
	private double seedProb;
	private double bugDensity;
	
	private FoodSpace food;
	private Grid2d world;
	
	private LinkedList bugList;
	private ActionGroup modelActions;
	private Schedule modelSchedule;
	
	FActionForEach actionForEach;    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	public ModelSwarm(Zone aZone)    // createBegin()
	{
		super(aZone);
		
		worldXSize = 80;
		worldYSize = 80;
		seedProb = 0.5;
		bugDensity = 0.1;
	}
	
	public List getBugList()
	{
		return bugList;
	}
	
	public Grid2d getWorld()
	{
		return world;
	}
	
	public FoodSpace getFood()
	{
		return food;
	}
	
	public Object buildObjects()
	{
		Bug aBug;
		int x, y;
		
		// super(getZone());
		
		food = new FoodSpace(getZone(), worldXSize, worldYSize);
		if(debug)
		{
			if(food == null)
				System.out.println("ModelSwarm.java: createObject(): food referens to null");
			else
				System.out.println("ModelSwarm.java: createObject(): food OK");
			System.out.println("ModelSwarm.java: food.seedFoodWithProb()");	
		}
		food.seedFoodWithProb(seedProb);
		
		if(debug)
			System.out.println("ModelSwarm.java: createObject(): creating world");
		world = new Grid2dImpl(getZone(), worldXSize, worldYSize);
		if(debug)
			if(world == null)
				System.out.println("ModelSwarm.java: createObject(): world referens to null");
			else
				System.out.println("ModelSwarm.java: createObject(): world OK");
		// world.setSizeX$Y(worldXSize, worldYSize);
		world.fillWithObject(null);
		
		bugList = new LinkedList();
		for(y = 0; y < worldYSize; y++)
			for(x = 0; x < worldXSize; x++)
				if(Math.random() < bugDensity)
				{
					aBug = new Bug(getZone());
					aBug.setWorld$Food(world, food);
					aBug.createEnd();
					aBug.setX$Y(x, y);
					
					bugList.addLast(aBug);
				}
		
		return this;
	}
	
	
	public Object buildActions()
	{
		// super(getZone());
		
		modelActions = new ActionGroupImpl(getZone());
		
		if(debug)
		{
			if(bugList == null)
				System.out.println("ModelSwarm: buildActions(): bugList == null");
			if(modelActions == null)
				System.out.println("ModelSwarm: buildActions(): modelActions == null");
		}
		
		try
		{
			modelActions.createActionForEach$message(bugList, new Selector(bugList.getClass(), "step", false)); 
		}
		catch(Exception e)
		{
			System.out.println("ModelSwarm: buildActions(): Selector exception ");
			// return null;
		}
		
		if(debug)
			System.out.println("ModelSwarm: buildActions(): creating modelSchedule");		
		modelSchedule = new ScheduleImpl(getZone(), 1);
		// modelSchedule.setRepeatInterval(1);
		modelSchedule.at$createAction(0, modelActions);
		
		return this;
	}
	
	
	public Activity activateIn(Swarm swarmContext)
	{
		super.activateIn(swarmContext);
		modelSchedule.activateIn(this);
				
		return this.getActivity();
	}

}

