// subor ObserverSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;

import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;

import swarm.objectbase.Swarm;
import swarm.objectbase.VarProbe;
import swarm.objectbase.MessageProbe;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.ZoomRasterImpl;

import swarm.analysis.EZGraph;
import swarm.analysis.EZGraphImpl;

import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;

import swarm.space.Value2dDisplay;
import swarm.space.Value2dDisplayImpl;
import swarm.space.Object2dDisplay;
import swarm.space.Object2dDisplayImpl;

import java.util.List;

public final class ObserverSwarm extends GUISwarmImpl
{
	private boolean debug = true;
	
	private int displayFrequency;
	
	private ActionGroup displayActions;				
  private Schedule displaySchedule;
	
	private ModelSwarm modelSwarm;
	
	private Colormap colorMap;
	private ZoomRaster worldRaster;
	
	private Value2dDisplay foodDisplay;
	private Object2dDisplay bugDisplay;
	
	public ObserverSwarm(Zone aZone)
	{
		super(aZone);
		displayFrequency = 1;
	}
	
	public Object buildObjects()
	{
		super.buildObjects();
		
		if(debug)
			System.out.println("ObserverSwarm: super.buildObjects() finished");
		modelSwarm = new ModelSwarm(getZone());
		Globals.env.createArchivedProbeDisplay(modelSwarm, "modelSwarm");
    Globals.env.createArchivedProbeDisplay(this, "observerSwarm");	
		getControlPanel().setStateStopped();
		if(debug)
			System.out.println("ObserverSwarm: modelSwarm.buildObjects()");
		modelSwarm.buildObjects();
		
		if(debug)
			System.out.println("ObserverSwarm: Setting colorMap");
		colorMap = new ColormapImpl(getZone());
		colorMap.setColor$ToName((byte)0, "black");
		colorMap.setColor$ToName((byte)1, "red");
		colorMap.setColor$ToName((byte)2, "green");
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: running constructor");
		worldRaster = new ZoomRasterImpl(getZone(), "worldRaster");
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setColormap()");
		worldRaster.setColormap(colorMap);
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setZoomFactor");
		worldRaster.setZoomFactor(4);
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setWidth$Hight()");
		worldRaster.setWidth$Height(80, 80);
		// worldRaster.setWidth$Height(modelSwarm.getWorld().getSizeX(), modelSwarm.getWorld().getSizeX());
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setWindowTitle()");
		worldRaster.setWindowTitle("Food World");
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: pack()");
		worldRaster.pack();
		
		if(debug)
		{
			System.out.println("ObserverSwarm: Setting foodDisplay");
			if(worldRaster == null)
				System.out.println("               worldRaster == null");
			if(colorMap == null)
				System.out.println("               colorMap == null");
			if(modelSwarm.getFood() == null)
				System.out.println("               modelSwarm.getFood() returns null");
		}
						
		foodDisplay = new Value2dDisplayImpl(getZone(), worldRaster, colorMap, modelSwarm.getFood());
		
		System.out.println("ObserverSwarm: Setting bugDisplay");
		try 
		{
    	bugDisplay = new Object2dDisplayImpl(getZone(), worldRaster, modelSwarm.getWorld(),
         new Selector(Class.forName("Bug"), "drawSelfOn", false)); 
					// bugDisplay = new Object2dDisplayImpl();
    } catch (Exception e) 
		{
			System.out.println("ObserverSwarm: Setting bugDisplay: Selector Exception");
    }
		
		System.out.println("ObserverSwarm: Setting bugDisplay: setObjectCollection()");
		bugDisplay.setObjectCollection(modelSwarm.getBugList());
		// bugDisplay.setDisplayMessage("drawSelfOn:");
		
		
	
		return this;
	}
	
	public Object buildActions()
	{
		super.buildActions();
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): modelSwarm.buildActions()");
		modelSwarm.buildActions();
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): create displayActions");
		displayActions = new ActionGroupImpl(getZone());
		try
		{
			displayActions.createActionTo$message(foodDisplay, new Selector(foodDisplay.getClass(), "display", false));
			displayActions.createActionTo$message(bugDisplay, new Selector(bugDisplay.getClass(), "display", false));
			displayActions.createActionTo$message(worldRaster, new Selector(worldRaster.getClass(), "drawSelf", false));
			// displayActions.createActionTo$message(actionCache, new Selector(actionCache.getClass(), "doTkEvents", false));
		}catch(Exception e)
		{
			System.out.println("ObserverSwarm: buildActions(): Selector exception");
		}
		
		displaySchedule = new ScheduleImpl(getZone());
		displaySchedule.setRepeatInterval(displayFrequency);
		displaySchedule.at$createAction(0, displayActions);
		
		return this;
	}
	
	public Activity activateIn(Swarm swarmContext)
	{
		super.activateIn(swarmContext);
		modelSwarm.activateIn(this);
		displaySchedule.activateIn(this);
		return getActivity();
	}

}

